module Main exposing (..)

import Browser
import Html exposing (..)
import Html.Attributes exposing (..)
import Html.Events exposing (..)
import Character exposing (Character)
import Sphere exposing (Sphere)
import BoundedInt exposing (BoundedInt)

main : Program () Model Msg
main = 
    Browser.document
        { init = init
        , view = view
        , update = update
        , subscriptions = subscriptions
        }

init : () -> ( Model, Cmd Msg )
init _ =
    (   { character = Character.new
        , modal = Nothing
        , modalValue = ""
        }
    , Cmd.none
    )

type alias Model = 
    { character : Character
    , modal : Maybe ModalType
    , modalValue : String
    }

type Msg 
    = OpenModal ModalType
    | CloseModal
    | ChangeModalValue String
    | SaveModalChange ModalType
    | ChangeSphere Sphere

type ModalType
    = Name

update : Msg -> Model -> ( Model, Cmd Msg )
update msg model = 
    case msg of
        OpenModal modalType -> 
            (   { model 
                | modal = ( Just modalType ) 
                , modalValue = modalValue modalType model.character
                }
            , Cmd.none
            )
        CloseModal ->
            ( { model | modal = Nothing }
            , Cmd.none
            )
        ChangeModalValue value ->
            ( { model | modalValue = value }
            , Cmd.none
            )
        SaveModalChange modalType ->
            case modalType of
                Name ->
                    (   { model 
                        | character = Character.changeName model.character model.modalValue 
                        , modal = Nothing
                        }
                    , Cmd.none
                    )
        ChangeSphere sphere ->
            (   { model
                | character = Character.changeSphere model.character sphere
                }
            , Cmd.none
            )

view : Model -> Browser.Document Msg
view model =
    { title = "Character Sheet"
    , body = 
        [ viewEditableText Name model.character.name
        , viewSpheres model.character.spheres
        , viewModal model
        ]
    }

viewEditableText : ModalType -> String -> Html Msg
viewEditableText modalType content =
    div []
        [ span [ class "icon-text" ]
            [ span [] [ text content ]
            , span 
                [ class "icon" 
                ] 
                [ i 
                    [ class "fas fa-pen"
                    , onClick ( OpenModal modalType ) 
                    ] []
                ]
            ]
        ]

viewSpheres : List Sphere -> Html Msg
viewSpheres spheres =
    div [] ( List.map viewSphere spheres )

viewSphere : Sphere -> Html Msg
viewSphere sphere =
    div []
        [ span [] [ text sphere.name ]
        , button 
            [ class "button is-small"
            , onClick ( ChangeSphere { sphere | dots = BoundedInt.change sphere.dots (sphere.dots.value - 1) } )
            ] [ text "-" ]
        , span [] [ text ( String.fromInt sphere.dots.value ) ]
        , button 
            [ class "button is-small"
            , onClick ( ChangeSphere { sphere | dots = BoundedInt.change sphere.dots (sphere.dots.value + 1) } )
            ] [ text "+" ]
        ]

viewModal : Model -> Html Msg
viewModal model =
    case model.modal of
        Nothing -> div [ class "modal" ] []
        Just modalType ->
            div [ class "modal is-active" ]
                [ div 
                    [ class "modal-background" 
                    , onClick CloseModal
                    ] []
                , div [ class "modal-card" ] 
                    [ header [ class "modal-card-head" ]
                        [ p [ class "modal-card-title" ] [ text ( "Change " ++ ( modalTitle modalType ) ) ]
                        ]
                    , section [ class "modal-card-body" ] 
                        [ p [] 
                            [ input 
                                [ class "input" 
                                , type_ "text"
                                , placeholder ( "Enter New " ++ ( modalTitle modalType ) )
                                , value model.modalValue
                                , onInput ( \x -> ChangeModalValue x )
                                ] []
                            ]
                        ]
                    , footer [ class "modal-card-foot" ] 
                        [ div [ class "buttons" ] 
                            [ button 
                                [ class "button is-success"
                                , onClick ( SaveModalChange modalType )
                                ] [ text "Save Changes" ]
                            , button 
                                [ class "button"
                                , onClick CloseModal
                                ] [ text "Cancel" ]
                            ]
                        ]
                    ]
                ]

modalTitle : ModalType -> String
modalTitle modalType =
    case modalType of
        Name -> "Character Name"

modalValue : ModalType -> Character -> String
modalValue modalType character =
    case modalType of
        Name -> character.name

subscriptions : Model -> Sub Msg
subscriptions _ =
  Sub.none

ariaLabel : String -> Attribute msg
ariaLabel value =
    Html.Attributes.attribute "aria-label" value